

    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
    *                                                                 *
    *   BROADCAST Chat Client 1.01                                    *
    *   Copyright  2004 Sebastian Pipping <webmaster@hartwork.org>   *
    *                                                                 *
    *   --> http://www.hartwork.org                                   *
    *                                                                 *
    *                                                                 *
    *   This source code is released under LGPL.                      *
    *   See LGPL.txt for details.                        2004-09-24   *
    *                                                                 *
    *   ___________________________________________________________   *
    *                                                                 *
    *   Big thanx to Ren Nyffenegger for the C++ Socket Class.       *
    *   http://www.adp-gmbh.ch/win/misc/sockets.html                  *
    *                                                                 *
    \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */



#include "devil_compile.h"
#include "Socket.h"

#include <windows.h>
#include <string.h>

#include "resource.h"


#define DEFAULT_PORT	9000


bool bRunning	= true;
bool bConnected	= false;


HWND wnd_in		= NULL;
HWND wnd_out	= NULL;
HWND wnd_chat	= NULL;

WNDPROC wndproc_in_old   = NULL;
WNDPROC wndproc_out_old  = NULL;


HFONT g_hFont = NULL;

SocketClient * sc = NULL;



/******************************************************************************************************
**  out_append                                                                                       **
******************************************************************************************************/

void out_append( const char * text )
{
	UINT uLen = SendMessage( wnd_out, WM_GETTEXTLENGTH, 0, 0 );
	SendMessage( wnd_out, EM_SETSEL, uLen, uLen );
	SendMessage( wnd_out, EM_REPLACESEL, 0, ( LPARAM )text );
}


/******************************************************************************************************
**  out_clear                                                                                        **
******************************************************************************************************/

void out_clear()
{
	SendMessage( wnd_out, WM_SETTEXT, 0, 0 );
}



/******************************************************************************************************
**  wndproc_in                                                                                       **
******************************************************************************************************/

BOOL CALLBACK wndproc_in( HWND hwnd, UINT message, WPARAM wp, LPARAM lp )
{
	switch( message )
	{
		case WM_KEYDOWN:
		{
			switch( wp )
			{
				case VK_RETURN:
				{
					char * sz = new char[ 1024 ];

					// Also allow passing newline-only
					UINT uLen = SendMessage( wnd_in, WM_GETTEXTLENGTH, 0, 0 );
					if( GetDlgItemText( wnd_chat, IDC_IN, sz, 1024 ) || !uLen )
					{
						SendMessage( wnd_in, WM_SETTEXT, 0, 0 );

						if( bConnected )
						{
							// Send input to server
							std::string in = sz;
							in += "\x0D\x0A";

							sc->SendBytes( in );
						}
						else
						{
							UINT port = DEFAULT_PORT;
							char host[ 256 ] = "";


							// Extract port if present
							char * last = sz + uLen - 1;	// last char
							char * s = last;

							int i = 1;
							while( ( *s >= '0' ) && ( *s <= '9' ) && ( s > sz ) )
							{
								s--;
							}

							if( ( s != last ) && ( *s == ':' ) )
							{
								// found
								port = atoi( ( char * )( s + 1 ) );
								strncpy( host, sz, ( s - sz ) );
							}
							else
							{
								// missing
								strcpy( host, sz );
							}


							out_append( "\x0D\x0A" );
							out_append( ">> Connecting to [ " );
							out_append( host );
							out_append( " ]..." );

							
							// Keep old socket open to prevent
							// winsock shutdown/reinit

							SocketClient * sc2 = NULL;
							if( sc != NULL )
							{
								sc2 = sc;
							}


							// Connect
							bool bError = false;

							try
							{
								sc = new SocketClient( host, port );
								if( sc2 != NULL )
								{
									delete sc2;
								}
							}
							catch( ... )
							{
								bError = true;
							}

							if( bError )
							{
								// Failed
								out_append( "\x0D\x0A" );
								out_append( ">> Could not connect to [ " );
								out_append( host );
								out_append( " ]." );
							}
							else
							{
								sc->SetBlocking( NonBlockingSocket );

								out_append( "\x0D\x0A" );
								out_append( ">> Connected." );
								out_append( "\x0D\x0A\x0D\x0A" );

								bConnected = true;
							}
						}
					}

					break;
				}
				case VK_ESCAPE:
				{
					EndDialog( wnd_chat, 0 );
					bRunning = false;

					break;
				}
			}

			break;
		}
	}
	return CallWindowProc( wndproc_in_old, hwnd, message, wp, lp );
}


/******************************************************************************************************
**  wndproc_out                                                                                      **
******************************************************************************************************/

BOOL CALLBACK wndproc_out( HWND hwnd, UINT message, WPARAM wp, LPARAM lp )
{
	switch( message )
	{
		case WM_KEYDOWN:
		{
			switch( wp )
			{
				case VK_ESCAPE:
				{
					EndDialog( wnd_chat, 0 );
					bRunning = false;

					break;
				}
			}

			break;
		}
	}

	return CallWindowProc( wndproc_out_old, hwnd, message, wp, lp );
}


/******************************************************************************************************
**  wndproc_chat                                                                                     **
******************************************************************************************************/

BOOL CALLBACK wndproc_chat( HWND hwnd, UINT message, WPARAM wp, LPARAM lp )
{
	switch( message )
	{
		case WM_INITDIALOG:
		{
			// Handles
			wnd_in   = GetDlgItem( hwnd, IDC_IN );
			wnd_out  = GetDlgItem( hwnd, IDC_OUT );
			wnd_chat = hwnd;


			// Apply font
			if( g_hFont != NULL )
			{
				SendMessage( wnd_in,  WM_SETFONT, (WPARAM)g_hFont, false );
				SendMessage( wnd_out, WM_SETFONT, (WPARAM)g_hFont, false );
			}


			// Set buffer size to 1MB
			SendMessage( wnd_in, EM_SETLIMITTEXT, 1000000, 0 );


			out_clear();
			out_append( ">> Please enter destination host." );


			// Exchance wndproc IN
			wndproc_in_old = ( WNDPROC )GetWindowLong( wnd_in, GWL_WNDPROC );
			if( wndproc_in_old != NULL )
			{
				SetWindowLong( wnd_in, GWL_WNDPROC, ( LONG )wndproc_in );
			}

			// Exchance wndproc OUT
			wndproc_out_old = ( WNDPROC )GetWindowLong( wnd_out, GWL_WNDPROC );
			if( wndproc_out_old != NULL )
			{
				SetWindowLong( wnd_out, GWL_WNDPROC, ( LONG )wndproc_out );
			}

			return TRUE;	// Sets focus
		}
		case WM_COMMAND:
		{
			if( wp == IDCANCEL )
			{
				EndDialog( hwnd, 0 );
				bRunning = false;
			}

			break;
		}
	}

	return 0;
}


/******************************************************************************************************
**  WinMain                                                                                          **
******************************************************************************************************/

int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE, LPSTR, int )
{
	g_hFont = CreateFont(	-12,							// Height Of Font
							0,								// Width Of Font
							0,								// Angle Of Escapement
							0,								// Orientation Angle
							FW_REGULAR,						// Font Weight
							FALSE,							// Italic
							FALSE,							// Underline
							FALSE,							// Strikeout
							ANSI_CHARSET,					// Character Set Identifier
							OUT_TT_PRECIS,					// Output Precision
							CLIP_DEFAULT_PRECIS,			// Clipping Precision
							ANTIALIASED_QUALITY,			// Output Quality
							FF_DONTCARE|DEFAULT_PITCH,		// Family And Pitch
							"Courier New" );				// Font Name

	
	HWND hwnd = CreateDialog( hInstance, MAKEINTRESOURCE( IDD_CHAT ), NULL, wndproc_chat );
	ShowWindow( hwnd, SW_SHOW );


	MSG msg;
	while( bRunning )
	{
		if( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) )
		{
			TranslateMessage( &msg );
			DispatchMessage( &msg );
		}
		else if( bConnected )
		{
			bool bError = false;
			std::string out;


			try
			{
				out = sc->ReceiveLine();
			}
			catch( ... )
			{
				bError = true;
			}

			if( bError )
			{
				bConnected = false;

				out_append( "\x0D\x0A" );
				out_append( ">> Disconnected." );

				continue;
			}

			
			if( !out.empty() )
			{
				// Cut off leading timestamp if present
				char * add = new char[ out.length() + 1 ];
				strcpy( add, out.c_str() );

				char * s = add;
				UINT uGo = 20;	// "YYYY-MM-DD HH:MM:SS "

				while( uGo && ( *s != '\0' ) )
				{
					uGo--;
					if(
						( *s == '-' ) ||
						( *s == ':' ) ||
						( *s == ' ' ) ||
						(
							( *s >= '0' ) &&
							( *s <= '9' )
						)
					)
					{
						s++;
					}
					else
					{
						break;
					}
				}

				if( !uGo )
				{
					out_append( s );
				}
				else
				{
					out_append( add );
				}
			}

			Sleep( 10 );
		}
	}


	DeleteObject( g_hFont );


	if( sc != NULL )
	{
		sc->Close();
		delete [] sc;
	}

	return 0;
}