

    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
    *                                                                 *
    *   BROADCAST Chat Client 1.01                                    *
    *   Copyright  2004 Sebastian Pipping <webmaster@hartwork.org>   *
    *                                                                 *
    *   --> http://www.hartwork.org                                   *
    *                                                                 *
    *                                                                 *
    *   This source code is released under LGPL.                      *
    *   See LGPL.txt for details.                        2004-09-24   *
    *                                                                 *
    *   ___________________________________________________________   *
    *                                                                 *
    *   Big thanx to Ren Nyffenegger for the C++ Socket Class.       *
    *   http://www.adp-gmbh.ch/win/misc/sockets.html                  *
    *                                                                 *
    \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */



/*

   Socket.cpp

   Copyright (C) 2002-2004 Ren Nyffenegger

   This source code is provided 'as-is', without any express or implied
   warranty. In no event will the author be held liable for any damages
   arising from the use of this software.

   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this source code must not be misrepresented; you must not
      claim that you wrote the original source code. If you use this source code
      in a product, an acknowledgment in the product documentation would be
      appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
      misrepresented as being the original source code.

   3. This notice may not be removed or altered from any source distribution.

   Ren Nyffenegger rene.nyffenegger@adp-gmbh.ch


   =======================================================================
   ==                   THIS VERSION HAS BEEN MODIFIED                  ==
   =======================================================================


   1.1															2004-09-14

   * [Socket::SetBlocking]			added
   * [Socket::ReceiveLine]			throws exception on close now
   * [SocketClient::SocketClient]	also accepts ip adresses now

   1.0

*/



#include <iostream>

#include "Socket.h"


#ifdef _DEBUG
#define ODS( x ) OutputDebugString( x )
#else
#define ODS( x )
#endif


using namespace std;

int Socket::nofSockets_= 0;



/******************************************************************************************************
**  Socket::Socket                                                                                   **
******************************************************************************************************/

Socket::Socket() : s_(0) {					ODS( "FUNCTION Socket::Socket()" );					// 01
  Start();
  // UDP: use SOCK_DGRAM instead of SOCK_STREAM
  s_ = socket(AF_INET,SOCK_STREAM,0);

  if (s_ == INVALID_SOCKET) {
    throw "INVALID_SOCKET";
  }

  refCounter_ = new int(1);
}


/******************************************************************************************************
**  Socket::Socket                                                                                   **
******************************************************************************************************/

Socket::Socket(SOCKET s) : s_(s) {			ODS( "FUNCTION Socket::Socket( SOCKET )" );			// 01
  Start();
  refCounter_ = new int(1);
};


/******************************************************************************************************
**  Socket::Socket                                                                                   **
******************************************************************************************************/

Socket::Socket(const Socket& o) {			ODS( "FUNCTION Socket::Socket( const Socket )" );	// 01
  refCounter_=o.refCounter_;
  (*refCounter_)++;
  s_         =o.s_;


  nofSockets_++;
}


/******************************************************************************************************
**  Socket::~Socket                                                                                  **
******************************************************************************************************/

Socket::~Socket() {							ODS( "FUNCTION Socket::~Socket" );					// 05
  if (! --(*refCounter_)) {
    Close();
    delete refCounter_;
  }


  --nofSockets_;
  if (!nofSockets_) End();
}


/******************************************************************************************************
**  Socket::Start                                                                                    **
******************************************************************************************************/

void Socket::Start() {						ODS( "FUNCTION Socket::Start" );					// 02
  if (!nofSockets_) {
    WSADATA info;
    if (WSAStartup(MAKEWORD(2,0), &info)) {

      throw "Could not start WSA";
    }
  }
  ++nofSockets_;
}


/******************************************************************************************************
**  Socket::End                                                                                      **
******************************************************************************************************/

void Socket::End() {						ODS( "FUNCTION Socket::End" );						// 07
  WSACleanup();
}


/******************************************************************************************************
**  Socket::Close                                                                                    **
******************************************************************************************************/

void Socket::Close() {						ODS( "FUNCTION Socket::Close" );					// 06
  closesocket(s_);
}


/******************************************************************************************************
**  Socket::operator =                                                                               **
******************************************************************************************************/

Socket& Socket::operator =(Socket& o) {		ODS( "FUNCTION Socket::operator =" );
  (*o.refCounter_)++;

  refCounter_=o.refCounter_;
  s_         =o.s_;

  nofSockets_++;

  return *this;
}


/******************************************************************************************************
**  Socket::SetBlocking                                                                              **
******************************************************************************************************/

void Socket::SetBlocking( TypeSocket type ) {
  u_long arg=(u_long)(type==NonBlockingSocket);
  ioctlsocket(s_,FIONBIO,&arg);
}


/******************************************************************************************************
**  Socket::ReceiveBytes                                                                             **
******************************************************************************************************/

std::string Socket::ReceiveBytes() {	//	ODS( "FUNCTION Socket::ReceiveBytes" );				// 04
  std::string ret;
  char buf[1024];

 for ( ; ; ) {
  u_long arg = 0;
  if (ioctlsocket(s_, FIONREAD, &arg) != 0)
   break;
  if (arg == 0)
   break;
  if (arg > 1024)
   arg = 1024;
  int rv = recv (s_, buf, arg, 0);
  if (rv <= 0)
   break;
  std::string t;
  t.assign (buf, rv);
  ret += t;
 }

  return ret;
}


/******************************************************************************************************
**  Socket::ReceiveLine                                                                              **
******************************************************************************************************/

std::string Socket::ReceiveLine() {		//	ODS( "FUNCTION Socket::ReceiveLine" );				// 04
  std::string ret;
   while (1) {
     char r;
     switch(recv(s_, &r, 1, 0)) {
       case 0: // not connected anymore;
          throw 0;
       case -1:
          if (errno == EAGAIN) {
             return ret;
          } else {
            // not connected anymore
            return "";
		  }
     }

     ret += r;
     if (r == '\n')  return ret;
   }
}


/******************************************************************************************************
**  Socket::SendLine                                                                                 **
******************************************************************************************************/

void Socket::SendLine(std::string s) {	//	ODS( "FUNCTION Socket::SendLine" );					// 04
  s += '\n';
  send(s_,s.c_str(),s.length(),0);
}


/******************************************************************************************************
**  Socket::SendBytes                                                                                **
******************************************************************************************************/

void Socket::SendBytes(const std::string& s) {	// ODS( "FUNCTION Socket::SendBytes" );			// 04
send(s_,s.c_str(),s.length(),0);
}


/******************************************************************************************************
**  SocketServer::SocketServer                                                                       **
******************************************************************************************************/

SocketServer::SocketServer(int port, int connections, TypeSocket type) {
  sockaddr_in sa;

  memset(&sa, 0, sizeof(sa));

  sa.sin_family = PF_INET;
  sa.sin_port = htons(port);
  s_ = socket(AF_INET, SOCK_STREAM, 0);
  if (s_ == INVALID_SOCKET) {
    throw "INVALID_SOCKET";
  }

  if(type==NonBlockingSocket) {
    u_long arg = 1;
    ioctlsocket(s_, FIONBIO, &arg);
  }

  /* bind the socket to the internet address */
  if (bind(s_, (sockaddr *)&sa, sizeof(sockaddr_in)) == SOCKET_ERROR) {
    closesocket(s_);
    throw "INVALID_SOCKET";
  }

  listen(s_, connections);
}


/******************************************************************************************************
**  SocketServer::Accept                                                                             **
******************************************************************************************************/

Socket* SocketServer::Accept() {
  SOCKET new_sock = accept(s_, 0, 0);
  if (new_sock == INVALID_SOCKET) {
          int rc = WSAGetLastError();
          if(rc==WSAEWOULDBLOCK) {
                  return 0; // non-blocking call, no request pending
          }
          else {
            throw "Invalid Socket";
      }
  }



  Socket* r = new Socket(new_sock);
  return r;
}


/******************************************************************************************************
**  SocketClient::SocketClient                                                                       **
******************************************************************************************************/

SocketClient::SocketClient(const std::string& host, int port) : Socket() {

	ODS( "FUNCTION SocketClient::SocketClient" );												// 03

  std::string error;
  hostent *he;

  // ip or name?
  unsigned long ip = inet_addr(host.c_str());
  if( ip != INADDR_NONE )
  {
    // ip
    he = gethostbyaddr((char *)&ip,4,AF_INET);
  }
  else
  {
    // name
    he = gethostbyname(host.c_str());
  }

  if (he == 0) {
    error = strerror(errno);
    throw error;
  }

  sockaddr_in addr;
  addr.sin_family = AF_INET;
  addr.sin_port = htons(port);
  addr.sin_addr = *((in_addr *)he->h_addr);
  memset(&(addr.sin_zero), 0, 8);

  if (::connect(s_, (sockaddr *) &addr, sizeof(sockaddr))) {
    error = strerror(WSAGetLastError());
    throw error;
  }
}


/******************************************************************************************************
**  SocketSelect::SocketSelect                                                                       **
******************************************************************************************************/

SocketSelect::SocketSelect(Socket const * const s1, Socket const * const s2, TypeSocket type) {
  FD_ZERO(&fds_);
  FD_SET(const_cast<Socket*>(s1)->s_,&fds_);
  if(s2) {
    FD_SET(const_cast<Socket*>(s2)->s_,&fds_);
  }


  TIMEVAL tval;
  tval.tv_sec  = 0;
  tval.tv_usec = 1;


  TIMEVAL *ptval;
  if(type==NonBlockingSocket) {
    ptval = &tval;
  }
  else {
    ptval = 0;
  }


  if (select (0, &fds_, (fd_set*) 0, (fd_set*) 0, ptval)
      == SOCKET_ERROR) throw "Error in select";
}


/******************************************************************************************************
**  SocketSelect::Readable                                                                           **
******************************************************************************************************/

bool SocketSelect::Readable(Socket const * const s) {
  if (FD_ISSET(s->s_,&fds_)) return true;
  return false;
}